from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from entangled.forms import EntangledModelFormMixin

from djangocms_frontend import settings
from djangocms_frontend.fields import DeviceChoiceField, IconGroup
from djangocms_frontend.helpers import insert_fields
from djangocms_frontend.settings import DEVICE_CHOICES

AUTO_SIZE = (("auto", _("Auto")),)


class DivSelectWidget(forms.Select):
    """Select widget contained in a div for simple styling purposes"""

    template_name = "djangocms_frontend/admin/widgets/select.html"

    class Media:
        css = {"all": ("djangocms_frontend/css/div_select.css",)}


class SizeSideWidget(forms.MultiWidget):  # lgtm [py/missing-call-to-init]
    """2 component widget allowing to choose a side and the size of a spacing"""

    def __init__(self, **kwargs):
        self.property = kwargs.pop("property")
        self.side_choices = kwargs.pop("side_choices")
        super().__init__(
            [
                IconGroup(
                    choices=[
                        (self.property + side, verbose)
                        for side, verbose in self.side_choices
                    ]
                ),
                DivSelectWidget(choices=kwargs.pop("size_choices")),
            ],
            **kwargs,
        )

    def decompress(self, value):
        if isinstance(value, str) and value:
            return value.split("-", 1)
        return ["", ""]  # [self.property+self.side_choices[0][0], ""]


class SpacingSizeSideField(forms.MultiValueField):  # lgtm [py/missing-call-to-init]
    """Field for spacing information using SizeSideWidget"""

    def __init__(self, **kwargs):
        size_choices = settings.EMPTY_CHOICE + kwargs.pop("size_choices")
        side_choices = kwargs.pop("side_choices")
        prop = kwargs.pop("property")
        kwargs.setdefault(
            "fields",
            (
                forms.ChoiceField(
                    choices=[(prop + side, verbose) for side, verbose in side_choices],
                ),
                forms.ChoiceField(choices=size_choices, required=False),
            ),
        )
        kwargs.setdefault(
            "widget",
            SizeSideWidget(
                property=prop,
                size_choices=size_choices,
                side_choices=side_choices,
            ),
        )
        kwargs.setdefault("require_all_fields", False)
        kwargs.setdefault("required", False)
        super().__init__(**kwargs)

    def compress(self, data_list):
        if data_list and data_list[1]:  # Size selected?
            return f"{data_list[0]}-{data_list[1]}"
        return ""

    def clean(self, value):
        if value[1] and not value[0]:
            raise ValidationError(
                _("Please choose a side to which the spacing should be applied."),
                code="incomplete",
            )
        return super().clean(value)


def get_spacing_classes(spacing_set, active_set=None):
    """Generates the necessary bootstrap spacing utility classes"""

    if active_set is None or len(active_set) == len(DEVICE_CHOICES):
        return [spacing for spacing in spacing_set if spacing[-1] != "-"]

    classes = []
    active = False
    for spacing in spacing_set:
        for device, __ in DEVICE_CHOICES:
            if (device in active_set) != active:
                active = device in active_set
                if device == "xs":
                    if spacing[-1] != "-":
                        classes.append(spacing)
                else:
                    left, right = spacing.rsplit("-", 1)
                    if right:
                        classes.append(f"{left}-{device}-{right if active else '0'}")
    return classes


class MarginMixin:
    blockname = _("Margin")

    def get_fieldsets(self, request, obj=None):
        return insert_fields(
            super().get_fieldsets(request, obj),
            (("margin_x", "margin_y"), "margin_devices"),
            block=None,
            position=-1,
            blockname=self.blockname,
        )

    def render(self, context, instance, placeholder):
        instance.add_classes(
            get_spacing_classes(
                [
                    instance.config[field]
                    for field in ("margin_x", "margin_y")
                    if field in instance.config and instance.config[field]
                ],
                instance.config.get("margin_devices", None),
            )
        )
        return super().render(context, instance, placeholder)


class PaddingMixin:
    blockname = _("Padding")

    def get_fieldsets(self, request, obj=None):
        blockname = self.blockname
        blockattrs = dict()
        fs = super().get_fieldsets(request, obj)
        for label, fields in fs:  # noqa
            if label == blockname:  # already included MarginMixin
                blockname = _("Padding")
                blockattrs = dict(classes=())
                break

        return insert_fields(
            fs,
            (("padding_x", "padding_y"), "padding_devices"),
            block=None,
            position=-1,
            blockname=blockname,
            blockattrs=blockattrs,
        )

    def render(self, context, instance, placeholder):
        instance.add_classes(
            get_spacing_classes(
                [
                    instance.config[field]
                    for field in ("padding_x", "padding_y")
                    if field in instance.config and instance.config[field]
                ],
                instance.config.get("padding_devices", None),
            )
        )
        return super().render(context, instance, placeholder)


class SpacingMixin(PaddingMixin, MarginMixin):
    blockname = _("Spacing")


class MarginFormMixin(EntangledModelFormMixin):
    class Meta:
        entangled_fields = {
            "config": [
                "margin_x",
                "margin_y",
                "margin_devices",
            ]
        }

    margin_x = SpacingSizeSideField(
        label=_("Horizontal margin"),
        property="m",
        size_choices=settings.SPACER_SIZE_CHOICES + AUTO_SIZE,
        side_choices=settings.SPACER_X_SIDES_CHOICES,
    )
    margin_y = SpacingSizeSideField(
        label=_("Vertical margin"),
        property="m",
        size_choices=settings.SPACER_SIZE_CHOICES + AUTO_SIZE,
        side_choices=settings.SPACER_Y_SIDES_CHOICES,
    )
    margin_devices = DeviceChoiceField(
        label=_("Apply margin on device"),
        required=False,
        help_text=_(
            "Select only devices on which the margin should be applied. On other devices "
            "larger than the first selected device the margin will be set to zero."
        ),
    )


class PaddingFormMixin(EntangledModelFormMixin):
    class Meta:
        entangled_fields = {
            "config": [
                "padding_x",
                "padding_y",
                "padding_devices",
            ]
        }

    padding_x = SpacingSizeSideField(
        label=_("Horizontal padding"),
        property="p",
        size_choices=settings.SPACER_SIZE_CHOICES,
        side_choices=settings.SPACER_X_SIDES_CHOICES,
    )
    padding_y = SpacingSizeSideField(
        label=_("Vertical padding"),
        property="p",
        size_choices=settings.SPACER_SIZE_CHOICES,
        side_choices=settings.SPACER_Y_SIDES_CHOICES,
    )
    padding_devices = DeviceChoiceField(
        label=_("Apply padding on device"),
        required=False,
        help_text=_(
            "Select only devices on which the padding should be applied. On other devices "
            "larger than the first selected device the padding will be set to zero."
        ),
    )


class SpacingFormMixin(MarginFormMixin, PaddingFormMixin):
    pass



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.contrib.accordion.cms_plugins import (
    AccordionItemPlugin,
    AccordionPlugin,
)
from djangocms_frontend.contrib.accordion.forms import AccordionForm, AccordionItemForm

from ..fixtures import TestFixture


class AccordionPluginTestCase(TestFixture, CMSTestCase):
    def test_plugin(self):
        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=AccordionPlugin.__name__,
            language=self.language,
        )
        parent.initialize_from_form(AccordionForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'id="parent-1"')

        plugin = add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=AccordionItemPlugin.__name__,
            language=self.language,
            config=dict(attributes={"class": "test-class"}),
        )
        plugin.initialize_from_form(AccordionItemForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="accordion-item">')
        self.assertTrue(
            (
                'class="collapse accordion-collapse test-class"'
                in response.content.decode("utf-8")
            )
            or (
                'class="accordion-collapse collapse test-class"'
                in response.content.decode("utf-8")
            )
            or (
                'class="accordion-collapse test-class collapse"'
                in response.content.decode("utf-8")
            )
            or (
                'class="collapse test-class accordion-collapse"'
                in response.content.decode("utf-8")
            )
            or (
                'class="test-class collapse accordion-collapse"'
                in response.content.decode("utf-8")
            )
            or (
                'class="test-class accordion-collapse collapse"'
                in response.content.decode("utf-8")
            ),
            f'<div class="collapse accordion-collapse test-class"> not found in {response.content.decode("utf-8")}',
        )
        self.assertContains(response, 'id="heading-')



#!/usr/bin/env python
from setuptools import find_packages, setup

from djangocms_frontend import __version__

REQUIREMENTS = [
    "Django>=2.2",
    "django-cms>=3.7",
    "django-filer>=1.7",
    "easy-thumbnails",
    "djangocms-attributes-field>=1",
    "djangocms-text-ckeditor>=3.1.0",
    "django-select2",
    "django-entangled>=0.5.4",
]

EXTRA_REQUIREMENTS = {
    "djangocms-icon": [
        "djangocms-icon>=1.4.0",
    ],
    "static-ace": [
        "djangocms-static-ace",
    ],
}

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Web Environment",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: BSD License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Framework :: Django",
    "Framework :: Django :: 2.2",
    "Framework :: Django :: 3.2",
    "Framework :: Django :: 4.0",
    "Framework :: Django :: 4.1",
    "Framework :: Django :: 4.2",
    "Framework :: Django CMS",
    "Framework :: Django CMS :: 3.8",
    "Framework :: Django CMS :: 3.9",
    "Framework :: Django CMS :: 3.10",
    "Framework :: Django CMS :: 3.11",
    "Framework :: Django CMS :: 4.0",
    "Framework :: Django CMS :: 4.1",
    "Topic :: Internet :: WWW/HTTP",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    "Topic :: Software Development",
    "Topic :: Software Development :: Libraries",
]

PROJECT_URLS = {
    "Documentation": "https://djangocms-frontend.readthedocs.io",
    "Release notes": "https://github.com/django-cms/djangocms-frontend/blob/master/CHANGELOG.rst",
    "Issues": "https://github.com/django-cms/djangocms-frontend/issues",
    "Source": "https://github.com/django-cms/djangocms-frontend",
}


setup(
    name="djangocms-frontend",
    version=__version__,
    author="fsbraun",
    author_email="fsbraun@gmx.de",
    maintainer="Django CMS Association and contributors",
    maintainer_email="info@django-cms.org",
    url="https://github.com/django-cms/djangocms-frontend",
    license="BSD-3-Clause",
    description="Adds abstract User Interface items as plugins to django CMS.",
    long_description=open("README.rst").read(),
    long_description_content_type="text/x-rst",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    extras_require=EXTRA_REQUIREMENTS,
    classifiers=CLASSIFIERS,
    project_urls=PROJECT_URLS,
    test_suite="run_tests.run",
)



from djangocms_frontend import models
from djangocms_frontend.helpers import get_related_object

from .base import SubcommandsCommand


class StaleReferences(SubcommandsCommand):
    help = "Prints all stale references in the djangocms frontend plugins"
    command_name = "stale_references"

    def handle(self, *args, **options):
        for ui_item in models.FrontendUIItem.objects.all():
            for key, value in ui_item.config.items():
                if isinstance(value, dict):
                    if "model" in value and "pk" in value:
                        obj = get_related_object(ui_item.config, key)
                        if obj is None:
                            msg = f"{ui_item.ui_item} (pk={ui_item.pk}) stale field {key}."
                            pages = ui_item.placeholder.page_set.all()
                            self.stdout.write(self.style.ERROR(msg))
                            if pages:
                                for page in pages:
                                    self.stdout.write(
                                        self.style.WARNING(
                                            f"... on page #{page.id}, at {page.get_absolute_url()}"
                                        )
                                    )
                            else:
                                self.stdout.write(
                                    self.style.WARNING(
                                        f"... in placeholder #{ui_item.placeholder.id}"
                                    )
                                )

        self.stdout.write(self.style.SUCCESS("Finished checking references"))



from django import forms
from django.utils.translation import gettext_lazy as _
from entangled.forms import EntangledModelForm

from ... import settings
from ...fields import AttributesFormField, TagTypeFormField
from ...models import FrontendUIItem
from .. import collapse

# TODO leaving this comment for now
# data-bs-toggle="collapse" data-bs-target="#collapseExample"
# aria-expanded="false" aria-controls="collapseExample">
# data-bs-target can also be classes
# data-bs-parent links to the wrapper collapse
# <div class="collapse" id="collapseExample">

mixin_factory = settings.get_forms(collapse)


class CollapseForm(mixin_factory("Collapse"), EntangledModelForm):
    """
    Component > "Collapse" Plugin
    https://getbootstrap.com/docs/5.0/components/collapse/
    """

    class Meta:
        model = FrontendUIItem
        entangled_fields = {
            "config": [
                "collapse_siblings",
                "attributes",
            ]
        }
        untangled_fields = ("tag_type",)

    collapse_siblings = forms.CharField(
        label=_("Siblings"),
        initial=".card",
        required=False,
        help_text=_("Element to be used to create accordions."),
    )
    attributes = AttributesFormField()
    tag_type = TagTypeFormField()


class CollapseTriggerForm(mixin_factory("CollapseTrigger"), EntangledModelForm):
    """
    Component > "Collapse Trigger" Plugin
    https://getbootstrap.com/docs/5.0/components/collapse/
    """

    class Meta:
        model = FrontendUIItem
        entangled_fields = {
            "config": [
                "trigger_identifier",
                "attributes",
            ]
        }
        untangled_fields = ("tag_type",)

    trigger_identifier = forms.SlugField(
        label=_("Unique identifier"),
        required=True,
        help_text=_("Identifier to connect trigger with container."),
    )
    attributes = AttributesFormField()
    tag_type = TagTypeFormField()


class CollapseContainerForm(mixin_factory("CollapseContainer"), EntangledModelForm):
    """
    Component > "Collapse Container" Plugin
    https://getbootstrap.com/docs/5.0/components/collapse/
    """

    class Meta:
        model = FrontendUIItem
        entangled_fields = {
            "config": [
                "container_identifier",
                "attributes",
            ]
        }
        untangled_fields = ("tag_type",)

    container_identifier = forms.SlugField(
        label=_("Unique identifier"),
        required=True,
        help_text=_("Identifier to connect trigger with container."),
    )
    attributes = AttributesFormField()
    tag_type = TagTypeFormField()



from django.test import TestCase

from djangocms_frontend.contrib.link.models import Link


class LinkModelTestCase(TestCase):
    def test_instance(self):
        instance = Link.objects.create(
            config=dict(name="Get it!", external_link="https://www.django-cms.com/")
        )
        self.assertEqual(str(instance), "Link (1)")
        self.assertEqual(
            instance.get_short_description(), "Get it! (https://www.django-cms.com/)"
        )



from django.conf import settings
from django.utils.translation import gettext_lazy as _

# The default grid size for Bootstrap 5 is 12. You can change this setting
# to whatever layout you require. We suggest that the value is at
# least devisable by 2, 3 and 4.
GRID_SIZE = getattr(
    settings,
    "DJANGOCMS_FRONTEND_GRID_SIZE",
    12,
)

# Bootstrap 5 provides more than 2 container types, .container and .container-fluid
# https://getbootstrap.com/docs/5.0/layout/grid/#no-gutters
GRID_CONTAINER_CHOICES = getattr(
    settings,
    "DJANGOCMS_FRONTEND_GRID_CONTAINERS",
    (
        ("container", _("Container")),
        ("container-fluid", _("Fluid container")),
        ("container-full", _("Full container")),
    ),
)

# Options for flexbox on the alignment of the grid
# https://flexbox.webflow.com/
GRID_ROW_VERTICAL_ALIGNMENT_CHOICES = (
    ("align-items-start", _("Align items start")),
    ("align-items-center", _("Align items center")),
    ("align-items-end", _("Align items end")),
)

GRID_ROW_HORIZONTAL_ALIGNMENT_CHOICES = (
    ("justify-content-start", _("Justify content start")),
    ("justify-content-center", _("Justify content center")),
    ("justify-content-end", _("Justify content end")),
    ("justify-content-around", _("Justify content around")),
    ("justify-content-between", _("Justify content between")),
)

GRID_COLUMN_ALIGNMENT_CHOICES = (
    ("align-self-start", _("Align self start")),
    ("align-self-center", _("Align self center")),
    ("align-self-end", _("Align self end")),
)



from importlib import import_module

from djangocms_frontend import settings

try:
    module = import_module(f"..{settings.framework}.responsive", __name__)
    ResponsiveFormMixin = module.ResponsiveFormMixin
    ResponsiveMixin = module.ResponsiveMixin
except ModuleNotFoundError:

    class ResponsiveMixin:
        pass

    class ResponsiveFormMixin:
        pass



from django.test import TestCase

from djangocms_frontend.contrib.image.forms import ImageForm
from djangocms_frontend.contrib.image.models import Image


class PictureModelTestCase(TestCase):
    def test_instance(self):
        instance = Image.objects.create().initialize_from_form(ImageForm)
        self.assertEqual(str(instance), "Image (1)")
        self.assertEqual(instance.get_short_description(), "<file is missing>")



import json

from django.forms.fields import JSONField, TextInput

from djangocms_frontend.contrib.icon.conf import ICON_LIBRARIES


class IconPickerWidget(TextInput):  # pragma: no cover
    class Media:
        js = ("djangocms_frontend/icon/vendor/assets/js/universal-icon-picker.min.js",)

    template_name = "djangocms_frontend/admin/widgets/icon_picker.html"

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        try:
            json_obj = json.loads(value) or {}
        except ValueError:
            json_obj = {}
        if isinstance(json_obj, str):
            # Probably djangocms_icon classes
            context["widget"]["preview"] = f'<i class="{json_obj}"></i>'
            context["widget"]["library"] = ""
        else:
            context["widget"]["preview"] = json_obj.get("iconHtml", "")
            context["widget"]["library"] = json_obj.get("library", "")
        context["icon_libraries"] = [
            (key, key.title(), value[0], value[1])
            for key, value in ICON_LIBRARIES.items()
        ]
        return context


class IconPickerField(JSONField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault("widget", IconPickerWidget)
        super().__init__(*args, **kwargs)



from django.utils.translation import gettext as _

from ...models import FrontendUIItem


class ListGroup(FrontendUIItem):
    """
    Components > "List Group" Plugin
    https://getbootstrap.com/docs/5.0/components/list-group/
    """

    class Meta:
        proxy = True
        verbose_name = _("List group")

    def get_short_description(self):
        text = ""
        if self.list_group_flush:
            text += ".list-group-flush"
        return text


class ListGroupItem(FrontendUIItem):
    """
    Components > "List Group Ite" Plugin
    https://getbootstrap.com/docs/5.0/components/list-group/
    """

    class Meta:
        proxy = True
        verbose_name = _("List group item")

    def get_short_description(self):
        text = []
        if self.list_context:
            text.append(f".list-group-item-{self.list_context}")
        if self.list_state:
            text.append(f".{self.list_state}")
        return " ".join(text)



from cms.models import CMSPlugin
from cms.utils.compat import DJANGO_3_0
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils.html import conditional_escape, mark_safe
from django.utils.translation import gettext

from djangocms_frontend.fields import TagTypeField
from djangocms_frontend.settings import FRAMEWORK_PLUGIN_INFO

if DJANGO_3_0:
    from django_jsonfield_backport.models import JSONField
else:
    JSONField = models.JSONField


class FrontendUIItem(CMSPlugin):
    """
    Generic plugin model to store all frontend items. Plugin-specific information is stored in a JSON field
    called "config".
    """

    class Meta:
        verbose_name = gettext("UI item")

    ui_item = models.CharField(max_length=30)
    tag_type = TagTypeField(blank=True)
    config = JSONField(default=dict, encoder=DjangoJSONEncoder)

    def __init__(self, *args, **kwargs):
        self._additional_classes = []
        super().__init__(*args, **kwargs)

    def __getattr__(self, item):
        """Makes properties of plugin config available as plugin properties."""
        if (
            item[0] != "_" and item in self.config
        ):  # Avoid infinite recursion trying to get .config from db
            return self.config[item]
        return super().__getattribute__(item)

    def __str__(self):
        if "__str__" in self.config:
            return self.config["__str__"]
        return f"{gettext(self.ui_item)} ({str(self.pk)})"

    def add_classes(self, *args):
        for arg in args:
            if arg:
                self._additional_classes += arg.split() if isinstance(arg, str) else arg

    def add_attribute(self, attr, value=None):
        attrs = self.config.get("attributes", {})
        if attr == "style" and attr in attrs:
            value += attrs[attr]
        attrs.update({attr: value})
        self.config["attributes"] = attrs

    def get_attributes(self):
        attributes = self.config.get("attributes", {})
        classes = set(attributes.get("class", "").split())
        classes.update(self._additional_classes)
        if classes:
            attributes["class"] = " ".join(classes)
        parts = (
            f'{item}="{conditional_escape(value)}"' if value else f"{item}"
            for item, value in attributes.items()
        )
        attributes_string = " ".join(parts)
        return mark_safe(" " + attributes_string) if attributes_string else ""

    def save(self, *args, **kwargs):
        self.ui_item = self.__class__.__name__
        return super().save(*args, **kwargs)

    def initialize_from_form(self, form=None):
        """Populates the config JSON field based on initial values provided by the fields of form"""
        if form is None:
            form = self.get_plugin_class().form
        if isinstance(form, type):  # if is class
            form = form()  # instantiate
        entangled_fields = getattr(
            getattr(form, "Meta", None), "entangled_fields", {}
        ).get("config", ())
        for field in entangled_fields:
            self.config.setdefault(
                field, {} if field == "attributes" else form[field].initial or ""
            )
        return self

    def get_short_description(self):
        """Plugin-specific short description (to be defined by subclasses)"""
        return ""

    @property
    def framework_info(self):
        return FRAMEWORK_PLUGIN_INFO.get(self.__class__.__name__, None)



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase
from django.http import HttpRequest

from djangocms_frontend.contrib.image.cms_plugins import ImagePlugin
from djangocms_frontend.contrib.image.forms import ImageForm, get_templates
from djangocms_frontend.helpers import first_choice

from ..fixtures import TestFixture
from ..helpers import get_filer_image


class PicturePluginTestCase(TestFixture, CMSTestCase):
    def setUp(self):
        super().setUp()
        self.image = get_filer_image()

    def tearDown(self):
        super().tearDown()
        self.image.delete()

    def test_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=ImagePlugin.__name__,
            language=self.language,
            config={
                "picture": {"pk": self.image.id, "model": "filer.Image"},
                "use_responsive_image": "yes",
            },
        )
        plugin.initialize_from_form(ImageForm)
        plugin.save()
        self.assertTrue(plugin.is_responsive_image)

        self.publish(self.page, self.language)
        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'class="img-fluid"')

        # test picture_fluid, picture_rounded and picture_thumbnail options
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=ImagePlugin.__name__,
            language=self.language,
            config={
                "image": {"pk": self.image.id, "model": "filer.Image"},
                "picture_fluid": False,
                "picture_rounded": True,
                "picture_thumbnail": True,
            },
        )
        plugin.initialize_from_form(ImageForm)
        plugin.save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ('class="img-thumbnail rounded"' in response.content.decode("utf-8"))
            or ('class="rounded img-thumbnail"' in response.content.decode("utf-8")),
            f'class="img-thumbnail rounded" not found in {response.content.decode("utf-8")}',
        )

    def test_image_form(self):
        request = HttpRequest()
        request.POST = {
            "template": first_choice(get_templates()),
            "picture": "",
            "external_picture": "https://www.django-cms.com/",
            "use_responsive_image": "yes",
            "margin_devices": ["xs"],
        }
        form = ImageForm(request.POST)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data["config"]["use_responsive_image"], "yes")

        request.POST.update(
            {
                "use_automatic_scaling": True,
                "use_no_cropping": True,
            }
        )
        form = ImageForm(request.POST)
        self.assertFalse(form.is_valid())



from django.db.models import Count

from djangocms_frontend.models import FrontendUIItem

from .base import SubcommandsCommand


class FrequencyAnalysis(SubcommandsCommand):
    help = "Migrates plugins djangocms_bootstrap4 to djangocms_frontend"
    command_name = "frequency_analysis"

    def handle(self, *args, **options):
        analysis = (
            FrontendUIItem.objects.values("ui_item")
            .annotate(count=Count("ui_item"))
            .order_by("-count")
        )
        for element in analysis:
            self.stdout.write(f"{element['ui_item']:20}\t{element['count']:6}")



from django.test import TestCase

from djangocms_frontend.contrib.accordion.forms import AccordionForm, AccordionItemForm
from djangocms_frontend.contrib.accordion.models import Accordion, AccordionItem


class AccordionModelTestCase(TestCase):
    def test_instance(self):
        instance = Accordion.objects.create()
        instance.initialize_from_form(AccordionForm).save()
        self.assertEqual(str(instance), "Accordion (1)")
        self.assertEqual(instance.get_short_description(), "(0 entries)")

        item = AccordionItem.objects.create()
        item.initialize_from_form(AccordionItemForm)
        item.config["accordion_item_header"] = "man-machine"
        self.assertEqual(str(item), "AccordionItem (2)")
        self.assertEqual(item.get_short_description(), "man-machine")



from cms.plugin_base import CMSPluginBase
from django.utils.encoding import force_str


class CMSUIPlugin(CMSPluginBase):
    render_template = "djangocms_frontend/html_container.html"
    change_form_template = "djangocms_frontend/admin/base.html"

    def __str__(self):
        return force_str(super().__str__())



from django.utils.translation import gettext as _

from ...models import FrontendUIItem


class Spacing(FrontendUIItem):
    """
    Utilities > "Spacing" Plugin
    https://getbootstrap.com/docs/5.0/utilities/spacing/
    """

    class Meta:
        proxy = True
        verbose_name = _("Spacing")

    def get_base_css_class(self):
        # Source: https://getbootstrap.com/docs/5.0/utilities/spacing/#notation
        # [...] format {property}{sides}-{size} for xs and
        # {property}{sides}-{breakpoint}-{size} for sm, md, lg, and xl.

        if not self.space_device or self.space_device == "xs":
            template = "{property}{sides}-{size}"
        else:
            template = "{property}{sides}-{breakpoint}-{size}"

        return template.format(
            property=self.space_property,
            sides=self.space_sides,
            breakpoint=self.space_device,
            size=self.space_size,
        )

    def get_short_description(self):
        return f"(.{self.get_base_css_class()})"


class Heading(FrontendUIItem):
    """
    Creates a heading (<h1>, <h2>, ...) and adds the entry to the table of contents (if an id is given)
    """

    class Meta:
        proxy = True
        verbose_name = _("Heading")

    def get_short_description(self):
        return f"({self.heading})"


class TableOfContents(FrontendUIItem):
    """
    Creates a table of contents of all headings processed BEFORE the plugin.
    If the table of contents is supposed to be at the top of the page, move the surrounding column
    using flexbox directives.
    """

    class Meta:
        proxy = True
        verbose_name = _("Table of contents")

    def get_short_description(self):
        return f"({self.id})"



from importlib import import_module

from djangocms_frontend import settings

try:
    module = import_module(f"..{settings.framework}.background", __name__)
    BackgroundFormMixin = module.BackgroundFormMixin
    BackgroundMixin = module.BackgroundMixin
except ModuleNotFoundError:

    class BackgroundMixin:
        pass

    class BackgroundFormMixin:
        pass



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.contrib.listgroup.cms_plugins import (
    ListGroupItemPlugin,
    ListGroupPlugin,
)
from djangocms_frontend.contrib.listgroup.forms import ListGroupForm, ListGroupItemForm

from ..fixtures import TestFixture


class ListGroupPluginTestCase(TestFixture, CMSTestCase):
    def test_list_group_plugin(self):
        add_plugin(
            placeholder=self.placeholder,
            plugin_type=ListGroupPlugin.__name__,
            language=self.language,
            config=dict(list_group_flush=False),
        )
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="list-group">')

        # test list_group_flush option
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=ListGroupPlugin.__name__,
            language=self.language,
            config=dict(list_group_flush=True),
        )
        plugin.initialize_from_form(ListGroupForm)
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            (
                '<div class="list-group list-group-flush">'
                in response.content.decode("utf-8")
            )
            or (
                '<div class="list-group-flush list-group">'
                in response.content.decode("utf-8")
            ),
            '<div class="list-group-flush list-group"> not in response',
        )

    def test_list_group_item_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=ListGroupItemPlugin.__name__,
            language=self.language,
        )
        plugin.initialize_from_form(ListGroupItemForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="list-group-item">')

        # test list_context and list_state options
        add_plugin(
            placeholder=self.placeholder,
            plugin_type=ListGroupItemPlugin.__name__,
            language=self.language,
            config=dict(
                list_context="primary",
                list_state="active",
            ),
        )
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "active")
        self.assertContains(response, "list-group-item-primary")



# Generated by Django 3.1.8 on 2022-01-23 21:37

from django.db import migrations

import djangocms_frontend.contrib.image.models
import djangocms_frontend.contrib.link.models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("djangocms_frontend", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Image",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
                "verbose_name": "Image",
            },
            bases=(
                djangocms_frontend.contrib.link.models.GetLinkMixin,
                djangocms_frontend.contrib.image.models.ImageMixin,
                "djangocms_frontend.frontenduiitem",
            ),
        ),
    ]



import os
from tempfile import mkdtemp

from django.core.files import File
from filer.models.filemodels import File as FilerFile
from filer.models.foldermodels import Folder as FilerFolder
from filer.models.imagemodels import Image as FilerImage
from filer.utils.compatibility import PILImage, PILImageDraw


# from https://github.com/divio/django-filer/blob/develop/tests/helpers.py#L46-L52
def create_image(mode="RGB", size=(800, 600)):
    """
    Creates a usable image file using PIL
    :returns: PIL Image instance
    """
    image = PILImage.new(mode, size)
    draw = PILImageDraw.Draw(image)
    x_bit, y_bit = size[0] // 10, size[1] // 10
    draw.rectangle((x_bit, y_bit * 2, x_bit * 7, y_bit * 3), "red")
    draw.rectangle((x_bit * 2, y_bit, x_bit * 3, y_bit * 8), "red")

    return image


def get_image(image_name="test_file.jpg"):
    """
    Creates and stores an image to the file system using PILImage

    :param image_name: the name for the file (default "test_file.jpg")
    :returns: dict {name, image, path}
    """
    image = create_image()
    image_path = os.path.join(
        mkdtemp(),
        image_name,
    )
    image.save(image_path, "JPEG")

    return {
        "name": image_name,
        "image": image,
        "path": image_path,
    }


def get_file(file_name="test_file.pdf"):
    """
    Creates and stores an arbitrary file into a temporary dir

    :param file_name: the name for the file (default "test_file.pdf")
    :returns: dict {name, image, path}
    """
    file_path = os.path.join(
        mkdtemp(),
        file_name,
    )
    data = open(file_path, "a")

    return {
        "name": file_name,
        "file": data,
        "path": file_path,
    }


def get_filer_image(image_name="test_file.jpg", name="", original_filename=True):
    """
    Creates and stores an image to filer and returns it

    :param image_name: the name for the file (default "test_file.jpg")
    :returns: filer image instance
    """
    image = get_image(image_name)
    filename = None
    if original_filename:
        filename = image.get("name")
    filer_file = File(
        open(image.get("path"), "rb"),
        name=image.get("name"),
    )
    filer_object = FilerImage.objects.create(
        original_filename=filename,
        file=filer_file,
        name=name,
    )

    return filer_object


def get_filer_file(file_name="test_file.pdf", folder=None):
    """
    Creates and stores a file to filer and returns it

    :param file_name: the name for the file (default "test_file.pdf")
    :param folder: optionally provide a folder instance
    :returns: filer file instance
    """
    data = get_file(file_name)
    filer_file = File(
        open(data.get("path"), "rb"),
        name=data.get("name"),
    )
    filer_object = FilerFile.objects.create(
        original_filename=data.get("name"),
        file=filer_file,
        folder=folder,
    )

    return filer_object


def get_filer_folder(folder_name="test_folder", parent=None):
    """
    Creates and returns a filer folder

    :param folder_name: the name of the folder to be used (default "test_folder")
    :param parent: optionally provide a parent folder
    :returns: filer folder instance
    """
    filer_object = FilerFolder.objects.create(
        parent=parent,
        name=folder_name,
    )

    return filer_object



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.contrib.collapse.cms_plugins import (
    CollapseContainerPlugin,
    CollapsePlugin,
    CollapseTriggerPlugin,
)
from djangocms_frontend.contrib.collapse.forms import (
    CollapseContainerForm,
    CollapseForm,
    CollapseTriggerForm,
)

from ..fixtures import TestFixture


class CollapsePluginTestCase(TestFixture, CMSTestCase):
    def test_collapse_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CollapsePlugin.__name__,
            language=self.language,
        )
        plugin.initialize_from_form(CollapseForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'data-bs-children=".card"')
        self.assertContains(response, 'role="tablist"')

    def test_collapse_trigger_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CollapseTriggerPlugin.__name__,
            language=self.language,
            config=dict(
                trigger_identifier=10,
            ),
        )
        plugin.initialize_from_form(CollapseTriggerForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'aria-controls="10"')
        self.assertContains(response, 'data-bs-target="#10"')
        self.assertContains(response, 'id="trigger-10"')

    def test_collapse_container_plugin(self):
        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CollapsePlugin.__name__,
            language=self.language,
        )
        parent.initialize_from_form(CollapseForm).save()

        plugin = add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=CollapseContainerPlugin.__name__,
            language=self.language,
            config=dict(
                container_identifier=10,
            ),
        )
        plugin.initialize_from_form(CollapseContainerForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'aria-labelledby="trigger-10"')
        self.assertContains(response, "10")



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.contrib.jumbotron.cms_plugins import JumbotronPlugin

from ..fixtures import TestFixture


class JumbotronPluginTestCase(TestFixture, CMSTestCase):
    def test_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=JumbotronPlugin.__name__,
            language=self.language,
        )
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "container")

        # fluid option
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=JumbotronPlugin.__name__,
            language=self.language,
            config=dict(
                jumbotron_fluid=True,
            ),
        )
        plugin.full_clean()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "container-fluid")



from django.conf import settings
from django.utils.translation import gettext_lazy as _

LINK_CHOICES = (
    ("link", _("Link")),
    ("btn", _("Button")),
)

LINK_SIZE_CHOICES = (
    ("btn-sm", _("Small")),
    ("", _("Medium")),
    ("btn-lg", _("Large")),
)

USE_LINK_ICONS = getattr(
    settings,
    "DJANGOCMS_BOOTSTRAP5_USE_ICONS",
    True,
)

TARGET_CHOICES = (
    ("_blank", _("Open in new window")),
    ("_self", _("Open in same window")),
    ("_parent", _("Delegate to parent")),
    ("_top", _("Delegate to top")),
)



from importlib import import_module

from cms.forms.utils import get_page_choices
from cms.models import Page
from django.conf import settings as django_settings
from django.contrib.admin import site
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import FieldError, ObjectDoesNotExist
from django.utils.encoding import force_str
from django.utils.html import mark_safe

LINK_MODELS = getattr(django_settings, "DJANGOCMS_FRONTEND_LINK_MODELS", [])


def create_querysets(link_models):
    querysets = []
    for item in link_models:
        if item["class_path"] != "cms.models.Page":
            # CMS pages are collected using a cms function to preserve hierarchy
            section = item["type"]
            parts = item["class_path"].rsplit(".", 1)
            cls = getattr(import_module(parts[0]), parts[1])
            queryset = cls.objects

            if "manager_method" in item:
                queryset = getattr(queryset, item["manager_method"])()

            if "filter" in item:
                for k, v in item["filter"].items():
                    try:
                        # Attempt to execute any callables in the filter dict.
                        item["filter"][k] = v()
                    except TypeError:
                        # OK, it wasn't a callable, so, leave it be
                        pass
                queryset = queryset.filter(**item["filter"])
            else:
                if "manager_method" not in item:
                    queryset = queryset.all()
            if "order_by" in item:
                queryset = queryset.order_by(item["order_by"])
            querysets.append((section, queryset, item.get("search", None), cls))
    return querysets


_querysets = create_querysets(LINK_MODELS)


def get_object_for_value(value):
    if isinstance(value, str) and "-" in value:
        type_id, obj_id = value.split("-", 1)
        try:
            content_type = ContentType.objects.get(id=type_id)
            return dict(
                model=f"{content_type.app_label}.{content_type.model}",
                pk=int(obj_id),
            )
        except (ObjectDoesNotExist, TypeError):
            pass
    return None


def get_link_choices(request, term="", lang=None, nbsp=""):
    global _querysets

    available_objects = []
    # Now create our list of cms pages
    type_id = ContentType.objects.get_for_model(Page).id
    for value, descr in get_page_choices(lang):
        if isinstance(descr, list):
            available_objects.append(
                {
                    "text": value,
                    "children": [
                        dict(
                            id=f"{type_id}-{page}",
                            text=mark_safe(name.replace("&nbsp;", nbsp)),
                        )
                        for page, name in descr
                        if not isinstance(term, str) or term.upper() in name.upper()
                    ],
                }
            )
        elif value and isinstance(value, int):
            available_objects.append(dict(id=f"{type_id}-{value}"))

    # Add list of additional non-cms pages
    for section, qs, search, cls in _querysets:
        objects = None
        model_admin = site._registry.get(cls, None)
        if search:
            try:
                objects = qs.filter(**{search + "__icontains": term})
            except FieldError:
                pass
        if objects is None:
            objects = [
                item
                for item in qs.all()
                if (not isinstance(term, str)) or term.upper() in str(item).upper()
            ]
        if objects:
            type_class = ContentType.objects.get_for_model(objects[0].__class__)
            available_objects.append(
                {
                    "text": force_str(section),
                    "children": [
                        dict(id=f"{type_class.id}-{obj.id}", text=str(obj))
                        for obj in objects
                        if request is None
                        or model_admin
                        and model_admin.has_view_permission(request, obj=obj)
                    ],
                }
            )
    return available_objects


def get_choices(request, term="", lang=None):
    def to_choices(json):
        return list(
            (elem["text"], to_choices(elem["children"]))
            if "children" in elem
            else (elem["id"], elem["text"])
            for elem in json
        )

    return to_choices(get_link_choices(request, term, lang, "&nbsp;"))



from django.utils.translation import gettext as _

from ...models import FrontendUIItem


class Tab(FrontendUIItem):
    class Meta:
        proxy = True
        verbose_name = _("Tab")

    def get_short_description(self):
        text = f"({self.tab_type})"

        if self.tab_alignment:
            text += f" .{self.tab_alignment}"
        return text


class TabItem(FrontendUIItem):
    """
    Components > "Navs - Tab Item" Plugin
    https://getbootstrap.com/docs/5.0/components/navs/
    """

    class Meta:
        proxy = True
        verbose_name = _("Tab item")

    def get_short_description(self):
        return self.tab_title



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.contrib.tabs.cms_plugins import TabItemPlugin, TabPlugin
from djangocms_frontend.contrib.tabs.forms import TabForm, TabItemForm

from ..fixtures import TestFixture


class TabsPluginTestCase(TestFixture, CMSTestCase):
    def test_tab_plugin(self):
        add_plugin(
            placeholder=self.placeholder,
            plugin_type=TabPlugin.__name__,
            language=self.language,
        )
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "nav")

    def test_tab_item_plugin(self):
        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=TabPlugin.__name__,
            language=self.language,
        ).initialize_from_form(TabForm)
        parent.save()
        add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=TabItemPlugin.__name__,
            language=self.language,
            config=dict(tab_title="tab title"),
        ).initialize_from_form(TabItemForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="tab-content">')
        self.assertContains(response, "tab-pane")



class IconRenderMixin:
    render_template = "djangocms_frontend/bootstrap5/icon.html"

    def render(self, context, instance, placeholder):
        classes = instance.config.get("icon", {}).get("iconClass", "")
        instance.add_classes(*classes.split())
        context["icon_text"] = instance.config.get("icon", {}).get("iconText", "")
        size = instance.config.get("icon_size", "")
        if size:
            if size[-1] == "%":
                instance.add_attribute("style", f"font-size:{size};")
            else:
                instance.add_classes(size)
        if instance.config.get("icon_foreground", None):
            instance.add_classes(f"text-{instance.icon_foreground}")
        if instance.config.get("icon_rounded", False):
            instance.add_classes("text-center", "rounded", "rounded-circle")
            instance.add_attribute(
                "style",
                "display:inline-block;line-height:1.42em;height:1.42em;width:1.42em;",
            )
        return super().render(context, instance, placeholder)
